----------
-- Locals and Addon Declaration
----------
TickTo = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceDB-2.0", "AceConsole-2.0")
local L = AceLibrary("AceLocale-2.2"):new("TickTo")
local addon = TickTo
local font = GameFontNormal:GetFont()

L:RegisterTranslations("enUS", function() return {
	["A Micro-Management addon for Rogues! Hint: Use Alt+Drag to move the Frame!"] = true,
	["Energy"] = true,
	["Sets the Energy Values for the TickTo to display."] = true,
	["For example: /tickto energy 60 40 35 25"] = true,
} end)

-- Common rogue energy values needed for micromanagement
TickTo.defaults ={
	energyvalues = {
		60,
		40,
		35,
		25,
	}
}

-----------
-- Addon Initialization
-----------

function addon:OnInitialize()

	self:RegisterDB("TTDB", "TTPerCharDB")
	self:RegisterDefaults('profile', TickTo.defaults)
	
	TickTo.Options = {
		name = "TickTo",
		desc = L["A Micro-Management addon for Rogues! Hint: Use Alt+Drag to move the Frame!"],
		type = "group",
		args = {
			energy = {
				name = L["Energy"],
				type = "text",
				input = true,
				desc = L["Sets the Energy Values for the TickTo to display."],
				usage = L["For example: /tickto energy 60 40 35 25"],
				validate = function(...) for i = 1,4, select("#", ...) do if type(select(i, ...)) ~= "number" then return false end end return true end,
				get = function() return table.concat(self.db.profile.energyvalues, ', ') end,
				set = function(...) self.db.profile.energyvalues = {...}; self:EnergyUpdate() end,
			},
		}
	}
	self:RegisterChatCommand({"/tickto"}, TickTo.Options)

end

function addon:OnEnable()
	local class, enclass = UnitClass("player")
	if enclass == "ROGUE" then
		self:RegisterBucketEvent("UNIT_ENERGY", 0.2, "EnergyUpdate")
		self:CreateFrame()
		self:EnergyUpdate()
		self:RestorePosition()
	else
		DisableAddOn("TickTo")
	end
end

-----------
-- GUI Creation
-----------

function addon:CreateFrame()

	if self.frame then
		return
	end

	local f = CreateFrame("Frame", "TickToFrame", UIParent)
	self.frame = f

	f:SetBackdrop({
		bgFile = "Interface\\ChatFrame\\ChatFrameBackground", tile = true, tileSize = 16,
		edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", edgeSize = 16,
		insets = {left = 4, right = 4, top = 4, bottom = 4},
	})
	
	if IsAddOnLoaded("ClearFont") or IsAddOnLoaded("ClearFont2")
		then
			f:SetWidth(36)
		else
			f:SetWidth(40)
		end
	f:SetHeight(50)
	f:EnableMouse(true)
	f:SetMovable(true)
	f:RegisterForDrag"LeftButton"
	f:SetScript("OnDragStart", function() if not InCombatLockdown() and IsAltKeyDown() then this:StartMoving() end end)
	f:SetScript("OnDragStop", function() if not InCombatLockdown() then this:StopMovingOrSizing() self:SavePosition() end end)

	f:SetBackdropColor(0, 0, 0)
	f:SetBackdropBorderColor(0, 0, 0)
	f:SetPoint("CENTER", UIParent)

	for k, v in ipairs( self.db.profile.energyvalues ) do

		local offset = 6 - (k * 10)
	
		local ff = f:CreateFontString(nil,"OVERLAY")
		f[k] = ff
		ff:SetFont(font, 11)
		ff:SetWidth(27)
		if IsAddOnLoaded("ClearFont") or IsAddOnLoaded("ClearFont2")
		then
			ff:SetWidth(36)
		else
			ff:SetWidth(40)
		end
		ff:SetText("")
		ff:SetJustifyH("LEFT")
		ff:ClearAllPoints()
		ff:SetPoint("TOPLEFT", f, "TOPLEFT", 4, offset )
		
	end
	
	f:Show()

end

function addon:EnergyUpdate()

	for k, v in ipairs( self.db.profile.energyvalues ) do
		self:UpdateTicks(k,v)
	end

end

function addon:UpdateTicks(k,v)

	local numticks
	local amount = UnitMana("player")

	numticks = ceil( (v - amount) / 20 )
	
	       	if UnitIsDeadOrGhost("player") then 
			numticks = "|cffffffff"..v..":|r |cffff00000|r"
		elseif numticks <= 0 then
			numticks = "|cffffffff"..v..":|r |cff00ff000|r"
		elseif numticks == 1 then
			numticks = "|cffffffff"..v..":|r |cffff66001|r"
		else
			numticks = "|cffffffff"..v..":|r |cffff00002|r"
		end
		
	self.frame[k]:SetText(numticks)

end

function addon:SavePosition()

	local f = self.frame

	self.db.profile.posx = f:GetLeft()
	self.db.profile.posy = f:GetTop()

end

function addon:RestorePosition()
	
	local f = self.frame
	local x = self.db.profile.posx
	local y = self.db.profile.posy

	if not x or not y then return end

	f:ClearAllPoints()
	f:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", x, y)

end